@@ -84,7 +84,7 @@ def train_from_folder(
     generate = False,
     generate_interpolation = False,
     hamburger_res_layers = [32],
-    use_evonorm = False,
+    use_groupnorm = False,
     interpolation_num_steps = 100,
     save_frames = False,
     num_image_tiles = 8,
@@ -95,7 +95,7 @@ def train_from_folder(
     multi_gpus = False,
     calculate_fid_every = None,
     seed = 42,
-    log = False
+    activation = 'leaky_relu'
 ):
     model_args = dict(
         name = name,
@@ -104,7 +104,7 @@ def train_from_folder(
         batch_size = batch_size,
         gradient_accumulate_every = gradient_accumulate_every,
         hamburger_res_layers = cast_list(hamburger_res_layers),
-        use_evonorm = use_evonorm,
+        use_groupnorm = use_groupnorm,
         image_size = image_size,
         fmap_max = fmap_max,
         transparent = transparent,
@@ -116,7 +116,7 @@ def train_from_folder(
         aug_types = cast_list(aug_types),
         dataset_aug_prob = dataset_aug_prob,
         calculate_fid_every = calculate_fid_every,
-        log = log
+        activation = activation
     )
 
     if generate:
